@@ -55,6 +55,10 @@ class Simulate:
                                 'sea':[]}}
 
         self.metrics = {}
+        self.stats = np.zeros(4)
+        self.moving_std = np.zeros(4)
+        self.moving_mean  = np.zeros(4)
+        self.M2 = 0
 
 
     def reset(self):
@@ -69,7 +73,10 @@ class Simulate:
                           'B':{'land':[],
                                 'air':[],
                                 'sea':[]}}
-
+        self.stats = np.zeros(4)
+        self.moving_std = np.zeros(4)
+        self.moving_mean  = np.zeros(4)
+        self.M2 = 0
         self.metrics = {}
 
 
@@ -96,6 +103,38 @@ class Simulate:
         else:
             return None
 
+    def running_stats(self, status, eps=0.5):
+        self.stats[status] += 1
+        N = (self.cur_n + 1)
+        x = self.stats / N
+
+        self.moving_std  = self.moving_std + (x-self.moving_mean)**2
+        if N == 1:
+            self.moving_mean = x
+            return False
+
+
+        delta = x - self.moving_mean
+        self.moving_mean += delta / N
+
+        delta2 = x - self.moving_mean
+        self.M2 += delta * delta2
+        (mean, variance, sampleVariance) = (self.moving_mean, self.M2 / N, self.M2 / (N - 1))
+
+        if np.max(sampleVariance)**(1/2) < 0.01 and self.cur_n > 250:
+            self.moving_stats_last = self.moving_mean[:]
+            return True
+        else:
+            self.moving_stats_last = self.moving_mean[:]
+            return False
+
+
+
+
+
+
+
+
     def run(self, combat_system: CombatSystem, config=None) -> bool:
         troops_a, troops_b = 0, 0
         for T in ['sea', 'land', 'air']:
@@ -119,10 +158,13 @@ class Simulate:
 
 
         for n in tqdm(range(self.N)):
+            self.cur_n = n
             battle = Battle(copy.copy(self.army_a), 
                             copy.copy(self.army_b),
                             config)
             status = battle.run(combat_system=combat_system)
+            #print(status)
+            abrt = self.running_stats(status)
             self.statistics.append(status)
 
             for side in ['A', 'B']:
@@ -130,6 +172,8 @@ class Simulate:
                     self.survivors[side][type].append(battle.army[side].units[type])
                     if type == 'sea' and battle.army[side].submerged > 0:
                         self.survivors[side][type][-1][0] += battle.army[side].submerged
+            if abrt:
+                break
         return True
 
 
@@ -138,7 +182,7 @@ class Simulate:
         idx, count = np.unique(self.statistics, return_counts=True)
         stats = np.zeros(4)
         stats[idx] = count
-        stats /= self.N
+        stats /= (self.cur_n+1)
         self.statistics = np.array(self.statistics)
 
         self.survivors_a_ground = np.array(self.survivors['A'][self.battle_type])
@@ -255,16 +299,16 @@ class Simulate:
             str += f"{'AIR Red':<10}"
             buf.write(str+os.linesep)
             return buf.getvalue()
-       
+        N = self.cur_n+1
         report.write(f"Results:{os.linesep}")
-        report.write(f"{'A Won:':<20}{games_won_a / self.N:.2f}{os.linesep}")
-        report.write(f"{'B Won:':<20}{games_won_b / self.N:.2f}{os.linesep}") 
-        report.write(f"{'Draw:':<20}{games_draw / self.N:.2f}{os.linesep}")
-        report.write(f"{'Mutual Annihilation:':<20}{games_won_none / self.N:.2f}{os.linesep}")
+        report.write(f"{'A Won:':<20}{games_won_a / N:.2f}{os.linesep}")
+        report.write(f"{'B Won:':<20}{games_won_b / N:.2f}{os.linesep}") 
+        report.write(f"{'Draw:':<20}{games_draw /   N:.2f}{os.linesep}")
+        report.write(f"{'Mutual Annihilation:':<20}{games_won_none / N:.2f}{os.linesep}")
 
         if games_won_a > 0:
             report.write("++++++++++++++++++++++++++++++++++++++++++++"+os.linesep)
-            report.write(f"Army A Won - {games_won_a/self.N:.2f}{os.linesep}")
+            report.write(f"Army A Won - {games_won_a/N:.2f}{os.linesep}")
             report.write(print_legend())
             report.write(print_avg_surv(self.metrics['avg_surv_a'], id='AVG'))
             report.write("--------------------------------------------------------------"+os.linesep)
@@ -272,7 +316,7 @@ class Simulate:
 
         if games_won_b > 0:
             report.write("++++++++++++++++++++++++++++++++++++++++++++"+os.linesep)
-            report.write(f'Army B Won - {games_won_b/self.N:.2f}{os.linesep}')
+            report.write(f'Army B Won - {games_won_b/N:.2f}{os.linesep}')
             report.write(print_legend())
             report.write(print_avg_surv(self.metrics['avg_surv_b'], id='AVG'))
             report.write("--------------------------------------------------------------"+os.linesep)
@@ -280,7 +324,7 @@ class Simulate:
 
         if games_draw > 0:
             report.write("++++++++++++++++++++++++++++++++++++++++++++"+os.linesep)
-            report.write(f'Draw - {games_draw/self.N:.2f}{os.linesep}')
+            report.write(f'Draw - {games_draw/N:.2f}{os.linesep}')
             report.write(print_legend())
             report.write("--------------------------------------------------------------"+os.linesep)
             report.write(print_avg_surv(self.metrics['avg_draw_a'], id='A - avg'))
